@@ -1,5 +1,5 @@
 /*
- * "$Id: print-ps.c,v 1.101 2008/08/18 02:00:18 rlk Exp $"
+ * "$Id: print-ps.c,v 1.105 2012/01/19 13:25:40 m0m Exp $"
  *
  *   Print plug-in Adobe PostScript driver for the GIMP.
  *
@@ -65,25 +65,25 @@ static void	ps_ascii85(const stp_vars_t *, unsigned short *, int, int);
 static const stp_parameter_t the_parameters[] =
 {
   {
-    "PPDFile", N_("PPDFile"), N_("Basic Printer Setup"),
+    "PPDFile", N_("PPDFile"), "Color=Yes,Category=Basic Printer Setup",
     N_("PPD File"),
     STP_PARAMETER_TYPE_FILE, STP_PARAMETER_CLASS_FEATURE,
     STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
-    "PageSize", N_("Page Size"), N_("Basic Printer Setup"),
+    "PageSize", N_("Page Size"), "Color=No,Category=Basic Printer Setup",
     N_("Size of the paper being printed to"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_CORE,
     STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
-    "ModelName", N_("Model Name"), N_("Basic Printer Setup"),
+    "ModelName", N_("Model Name"), "Color=Yes,Category=Basic Printer Setup",
     N_("PPD File Model Name"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_CORE,
     STP_PARAMETER_LEVEL_INTERNAL, 0, 0, STP_CHANNEL_NONE, 0, 0
   },
   {
-    "PrintingMode", N_("Printing Mode"), N_("Core Parameter"),
+    "PrintingMode", N_("Printing Mode"), "Color=Yes,Category=Core Parameter",
     N_("Printing Output Mode"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_CORE,
     STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
@@ -770,6 +770,121 @@ ps_external_options(const stp_vars_t *v)
   return answer;
 }
 
+/*
+ * 'ps_print_device_settings()' - output postscript code from PPD into the
+ * postscript stream.
+ */
+
+static void
+ps_print_device_settings(stp_vars_t *v)
+{
+  int i;
+  stp_parameter_list_t param_list = ps_list_parameters(v);
+  if (! param_list)
+    return;
+  stp_puts("%%BeginSetup\n", v);
+  for (i = 0; i < stp_parameter_list_count(param_list); i++)
+    {
+      const stp_parameter_t *param = stp_parameter_list_param(param_list, i);
+      stp_parameter_t desc;
+      stp_describe_parameter(v, param->name, &desc);
+      if (desc.is_active)
+	{
+	  switch (desc.p_type)
+	    {
+	    case STP_PARAMETER_TYPE_STRING_LIST:
+	    case STP_PARAMETER_TYPE_BOOLEAN:
+	      {
+		const char *val=NULL;
+		const char *defval=NULL;
+
+		/* If this is a bool parameter, set val to "True" or "False" - otherwise fetch from string parameter. */
+		if(desc.p_type==STP_PARAMETER_TYPE_BOOLEAN)
+		  {
+		    val=stp_get_boolean_parameter(v,desc.name) ? "True" : "False";
+		    defval=desc.deflt.boolean ? "True" : "False";
+		  }
+		else
+		  {
+		    val=stp_get_string_parameter(v,desc.name);
+		    defval=desc.deflt.str;
+		  }
+
+		/* We only include the option's code if it's set to a value other than the default. */
+		if(val && defval && (strcmp(val,defval)!=0))
+		  {
+		    if(m_ppd)
+		      {
+			/* If we have a PPD xml tree we hunt for the appropriate "option" and "choice"... */
+			stp_mxml_node_t *node=m_ppd;
+			node=stp_mxmlFindElement(node,node, "option", "name", desc.name, STP_MXML_DESCEND);
+			if(node)
+			  {
+			    node=stp_mxmlFindElement(node,node, "choice", "name", val, STP_MXML_DESCEND);
+			    if(node && node->child)
+			      {
+				if(node->child->value.opaque && (strlen(node->child->value.opaque)>1))
+				  {
+				    /* If we have opaque data for the child, we use %%BeginFeature and copy the code verbatim. */
+				    stp_puts("[{\n", v);
+				    stp_zprintf(v, "%%%%BeginFeature: *%s %s\n", desc.name, val);
+				    if(node->child->value.opaque)
+				      stp_puts(node->child->value.opaque,v);
+				    stp_puts("\n%%EndFeature\n", v);
+				    stp_puts("} stopped cleartomark\n", v);
+				  }
+				else
+				  {
+				    /* If we don't have code, we use %%IncludeFeature instead. */
+				    stp_puts("[{\n", v);
+				    stp_zprintf(v, "%%%%IncludeFeature: *%s %s\n", desc.name, val);
+				    if(node->child->value.opaque)
+				      stp_puts(node->child->value.opaque,v);
+				    stp_puts("} stopped cleartomark\n", v);
+				  }
+			      }
+			  }
+		      }
+		  }
+	      }
+	      break;
+	    case STP_PARAMETER_TYPE_INT:
+	      if(stp_get_int_parameter(v,desc.name)!=desc.deflt.integer)
+		{
+		  stp_puts("[{\n", v);
+		  stp_zprintf(v, "%%%%IncludeFeature: *%s %d\n", desc.name,
+			      stp_get_int_parameter(v, desc.name));
+		  stp_puts("} stopped cleartomark\n", v);
+		}
+	      break;
+	    case STP_PARAMETER_TYPE_DOUBLE:
+	      if(stp_get_float_parameter(v,desc.name)!=desc.deflt.dbl)
+		{
+		  stp_puts("[{\n", v);
+		  stp_zprintf(v, "%%%%IncludeFeature: *%s %f\n", desc.name,
+			      stp_get_float_parameter(v, desc.name));
+		  stp_puts("} stopped cleartomark\n", v);
+		}
+	      break;
+	    case STP_PARAMETER_TYPE_DIMENSION:
+	      if(stp_get_dimension_parameter(v,desc.name)!=desc.deflt.dimension)
+		{
+		  stp_puts("[{\n", v);
+		  stp_zprintf(v, "%%%%IncludeFeature: *%s %d\n", desc.name,
+			      stp_get_dimension_parameter(v, desc.name));
+		  stp_puts("} stopped cleartomark\n", v);
+		}
+	      break;
+	    default:
+	      break;
+	    }
+	}
+      stp_parameter_description_destroy(&desc);
+    }
+  stp_puts("%%EndSetup\n", v);
+  stp_parameter_list_destroy(param_list);
+}
+
 /*
  * 'ps_print()' - Print an image to a PostScript printer.
  */
@@ -867,10 +982,7 @@ ps_print_internal(stp_vars_t *v, stp_image_t *image)
   stp_puts("%%Orientation: Portrait\n", v);
   stp_puts("%%EndComments\n", v);
 
-#if 0
-  /* This is still not correct -- rlk 20070601 */
   ps_print_device_settings(v);
-#endif
 
  /*
   * Output the page...
@@ -1089,7 +1201,6 @@ ps_hex(const stp_vars_t *v,	/* I - File to print to */
   int		col;		/* Current column */
   static const char	*hex = "0123456789ABCDEF";
 
-
   col = 0;
   while (length > 0)
   {
@@ -1133,6 +1244,9 @@ ps_ascii85(const stp_vars_t *v,	/* I - File to print to */
   unsigned char	c[5];			/* ASCII85 encoded chars */
   static int	column = 0;		/* Current column */
 
+#define OUTBUF_SIZE 4096
+  unsigned char outbuffer[OUTBUF_SIZE+10];
+  int outp=0;
 
   while (length > 3)
   {
@@ -1144,35 +1258,44 @@ ps_ascii85(const stp_vars_t *v,	/* I - File to print to */
 
     if (b == 0)
     {
-      stp_putc('z', v);
+      outbuffer[outp++]='z';
       column ++;
     }
     else
     {
-      c[4] = (b % 85) + '!';
+      outbuffer[outp+4] = (b % 85) + '!';
       b /= 85;
-      c[3] = (b % 85) + '!';
+      outbuffer[outp+3] = (b % 85) + '!';
       b /= 85;
-      c[2] = (b % 85) + '!';
+      outbuffer[outp+2] = (b % 85) + '!';
       b /= 85;
-      c[1] = (b % 85) + '!';
+      outbuffer[outp+1] = (b % 85) + '!';
       b /= 85;
-      c[0] = b + '!';
+      outbuffer[outp] = b + '!';
 
-      stp_zfwrite((const char *)c, 5, 1, v);
+      outp+=5;
       column += 5;
     }
 
     if (column > 72)
     {
-      stp_putc('\n', v);
+      outbuffer[outp++]='\n';
       column = 0;
     }
 
+    if(outp>=OUTBUF_SIZE)
+	{
+		stp_zfwrite((const char *)outbuffer, outp, 1, v);
+		outp=0;
+	}
+
     data += 4;
     length -= 4;
   }
 
+  if(outp)
+    stp_zfwrite((const char *)outbuffer, outp, 1, v);
+
   if (last_line)
   {
     if (length > 0)
