@@ -1,5 +1,5 @@
 /*
- * "$Id: print-raw.c,v 1.42 2010/12/05 21:38:15 rlk Exp $"
+ * "$Id: print-raw.c,v 1.47 2015/09/09 23:57:32 speachy Exp $"
  *
  *   Print plug-in RAW driver for the GIMP.
  *
@@ -26,6 +26,17 @@
  * compile on generic platforms that don't support glib, gimp, gtk, etc.
  */
 
+/*
+ * To use this driver, we recommend this:
+ *
+ *   stp_set_driver(v, "raw-data-8");  // or raw_data-16 
+ *   stp_set_string_parameter(v, "PageSize", "Custom");
+ *   stp_set_page_height(v, HEIGHT);
+ *   stp_set_page_width(v, WIDTH);
+ *
+ * For further details, see  compute_thumbnail() in gutenprintui2/panel.c
+ */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -88,6 +99,12 @@ static const stp_parameter_t the_parameters[] =
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_CORE,
     STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
+  {
+    "PageSize", N_("Page Size"), "Color=No,Category=Basic Printer Setup",
+    N_("Size of the paper being printed to"),
+    STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_CORE,
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
+  },
 };
 
 static const int the_parameter_count =
@@ -138,14 +155,25 @@ raw_parameters(const stp_vars_t *v, const char *name,
       description->deflt.str =
 	stp_string_list_param(description->bounds.str, 0)->name;
     }
+  else if (strcmp(name, "PageSize") == 0)
+    {
+      int papersizes = stp_known_papersizes();
+      description->bounds.str = stp_string_list_create();
+      for (i = 0; i < papersizes; i++)
+	{
+	  /* All users of the raw drivers should use "Custom" PageSize
+	     and manually set page height/width! */
+	  const stp_papersize_t *pt = stp_get_papersize_by_index(i);
+	  stp_string_list_add_string(description->bounds.str,
+				     pt->name, gettext(pt->text));
+	}
+      description->deflt.str =
+	stp_string_list_param(description->bounds.str, 0)->name;
+    }
   else
     description->is_active = 0;
 }
 
-/*
- * 'escp2_imageable_area()' - Return the imageable area of the page.
- */
-
 static void
 raw_imageable_area(const stp_vars_t *v,
 		   int  *left,
@@ -191,9 +219,6 @@ raw_describe_output(const stp_vars_t *v)
   return "RGB";
 }
 
-/*
- * 'escp2_print()' - Print an image to an EPSON printer.
- */
 static int
 raw_print(const stp_vars_t *v, stp_image_t *image)
 {
@@ -239,7 +264,6 @@ raw_print(const stp_vars_t *v, stp_image_t *image)
 	  }
     }
 
-  stp_set_float_parameter(nv, "Density", 1.0);
   stp_set_boolean_parameter(nv, "SimpleGamma", 1);
   stp_channel_reset(nv);
   for (i = 0; i < ink_channels; i++)
